/* dispatch_loop.h - sourced by RX and TX microblocks */

/**************************************************************/
/* This file is adapted from examples in the Intel SDK and    */
/* parts of the code are covered by Intel copyright.  You     */
/* only have permission to use this file if you have obtained */
/* a copy of the SDK from Intel.                              */
/**************************************************************/

#ifndef	__DISPATCH_LOOP__H__
#define	__DISPATCH_LOOP__H__


#include <dl_system.h>


#endif			// __DISPATCH_LOOP__H__
